/*
 *  The Dropline Update Applet
 *
 *  Copyright 2012 Christian Hergert 
 *  Copyright 2012 Sasa Ostrouska <saxa@droplinegnome.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "dropline-status-icon.h"

#define DLG_ICON "../pixmaps/applet-idle.svg"
#define DLG_APPLET_VERSION "0.1"
#define DLG_APPLET_COPYRIGHT "(c) Sasa Ostrouska"
#define DLG_WEBSITE "http://www.droplinegnome.org"
#define DROPLINE_FILES "http://www.droplinegnome.org/files/3.8/DroplineFiles3.8"

G_DEFINE_TYPE(DroplineStatusIcon, dropline_status_icon, GTK_TYPE_STATUS_ICON)

struct _DroplineStatusIconPrivate
{
   /*
    * XXX: Put stuff you need during runtime here.
    */
   gpointer dummy;
};

static GtkMenuShell *dlg_menu = NULL;
static GtkStatusIcon *status_icon = NULL;
static GtkWidget *window = NULL;

static void
dlg_preferences_window (GtkStatusIcon *status_icon, gpointer data)
{
	g_print("You are in dlg_preferences_window()\n");
	window = gtk_dialog_new_with_buttons("dropline GNOME preferences",
							GTK_WINDOW(window),
							0,
							GTK_STOCK_CLOSE,
							GTK_RESPONSE_NONE,
							NULL);

	gtk_window_set_title(GTK_WINDOW(window), "dropline GNOME preferences");
	gtk_window_set_default_size(GTK_WINDOW(window), 480, 320);

	g_signal_connect(window, "response", G_CALLBACK(gtk_widget_destroy), NULL);
	g_signal_connect(window, "destroy", G_CALLBACK(gtk_widget_destroyed), &window);

	gtk_widget_show(window);

}

static void
dlg_about_window (GtkWidget *window, gpointer data)
{
	g_print("You are in dlg_about_window()\n");

	GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file(DLG_ICON, NULL);

	GtkWidget *about = gtk_about_dialog_new();
	gtk_about_dialog_set_program_name(GTK_ABOUT_DIALOG(about), "dropline GNOME applet"); 
	gtk_about_dialog_set_version(GTK_ABOUT_DIALOG(about), DLG_APPLET_VERSION);
	gtk_about_dialog_set_copyright(GTK_ABOUT_DIALOG(about), DLG_APPLET_COPYRIGHT);

	gtk_about_dialog_set_comments(GTK_ABOUT_DIALOG(about),
		       			"This is the dropline GNOME applet, used to show new updates.");

	gtk_about_dialog_set_website(GTK_ABOUT_DIALOG(about), DLG_WEBSITE);
	gtk_about_dialog_set_logo(GTK_ABOUT_DIALOG(about), pixbuf);
	g_object_unref(pixbuf), pixbuf = NULL; 

	g_signal_connect(about, "response", G_CALLBACK(gtk_widget_destroy), NULL);
	g_signal_connect(about, "destroy", G_CALLBACK(gtk_widget_destroyed), &about);

	gtk_dialog_run(GTK_DIALOG(about));
	/* gtk_widget_destory(about); */
}

static void
dlg_check_for_updates (GtkStatusIcon *status_icon, gpointer data)
{
	g_print("You clicked check_for_updates()\n");

	CURL *curl;
	CURLcode res;
	
	curl = curl_easy_init();

	if (curl) {
		curl_easy_setopt(curl, CURLOPT_URL, DROPLINE_FILES);
		res = curl_easy_perform(curl);

		if (res != CURLE_OK)
			fprintf(stderr, "curl_easy_perform() failed: %s\n", curl_easy_strerror(res));

		curl_easy_cleanup(curl);
	}
	return 0;
}

static void
destroy (GtkWidget *widget, gpointer data)
{
	gtk_main_quit ();
}



static void
dlg_notify_cb (GtkWidget *widget, gpointer data)
{
	notify_init("dropline GNOME applet");
	NotifyNotification *notify;
	GError *error = NULL;
	GtkWidget *text = (GtkWidget *)data;
	char buffer[128];

	const char *string = gtk_entry_get_text(GTK_ENTRY(text));

	snprintf(buffer, 128, "This is what you typed:\n%s\n", string);

	notify = notify_notification_new("dropline GNOME applet",
					 buffer,
					 DLG_ICON);

	notify_notification_add_action(notify, 
					"update", /* action id */
					"Start updating", /* This is what is wrote on the button */
					dlg_start_installer, /* Function name called when we press the button */
					NULL,
					NULL);

	notify_notification_show(notify, &error);
	notify_uninit();
}


static void 
dlg_start_installer(NotifyNotification *n, char *update, gpointer data)
{
	g_print("You cliecked the notification button !!!\n");
}

static void
dlg_popup (GtkStatusIcon *status_icon,
		guint button,
		guint activate_time,
		gpointer user_data)
{
	if (!dlg_menu)
	{
		GtkWidget *item;
		dlg_menu = gtk_menu_new ();

		item = gtk_menu_item_new_with_label("Check updates...");
		gtk_menu_shell_append(dlg_menu, item);
		g_signal_connect(G_OBJECT(item), "activate",
				/*G_CALLBACK(dlg_check_for_updates),*/
				G_CALLBACK(dlg_notify_cb),
				NULL);

		item = gtk_menu_item_new_with_label("Preferences");
		gtk_menu_shell_append(dlg_menu, item);
		g_signal_connect(G_OBJECT(item), "activate",
				G_CALLBACK(dlg_preferences_window),
				NULL);

		item = gtk_separator_menu_item_new();
		gtk_menu_shell_append(dlg_menu, item);

		item = gtk_menu_item_new_with_label("About");
		gtk_menu_shell_append(dlg_menu, item);
		g_signal_connect(G_OBJECT(item), "activate",
				G_CALLBACK(dlg_about_window),
				NULL);

		item = gtk_separator_menu_item_new();
		gtk_menu_shell_append(dlg_menu, item);

		item = gtk_menu_item_new_with_label("Quit");
		gtk_menu_shell_append(dlg_menu, item);
		g_signal_connect(G_OBJECT(item), "activate",
				G_CALLBACK(destroy),
				NULL);
	}

	gtk_widget_show_all(dlg_menu);

	gtk_menu_popup(GTK_MENU(dlg_menu), 
			NULL,
			NULL,
			gtk_status_icon_position_menu,
			status_icon,
			button,
			activate_time);
}

GtkStatusIcon *
dropline_status_icon_new (void)
{
   return g_object_new(DROPLINE_TYPE_STATUS_ICON,
                       "file", DLG_ICON,
                       NULL);
   /* when you want to show this, use gtk_status_icon_set_visible(status_icon, TRUE) */
}

static void
dropline_status_icon_finalize (GObject *object)
{
   DroplineStatusIconPrivate *priv;

   priv = DROPLINE_STATUS_ICON(object)->priv;

   /*
    * XXX: Free stuff in _DroplineStatusIconPrivate here.
    */

   G_OBJECT_CLASS(dropline_status_icon_parent_class)->finalize(object);
}

static void
dropline_status_icon_class_init (DroplineStatusIconClass *klass)
{
   GObjectClass *object_class;

   object_class = G_OBJECT_CLASS(klass);
   object_class->finalize = dropline_status_icon_finalize;
   g_type_class_add_private(object_class, sizeof(DroplineStatusIconPrivate));
}

static void
dropline_status_icon_init (DroplineStatusIcon *icon)
{
   icon->priv =
      G_TYPE_INSTANCE_GET_PRIVATE(icon,
                                  DROPLINE_TYPE_STATUS_ICON,
                                  DroplineStatusIconPrivate);

   /*
    * XXX: Initialize anything you need here, like perhaps setting
    *      the icon.
    */
}

int main( int argc,
	  char* argv[] )
{
	GtkWidget *window = NULL;

	gtk_init( &argc, &argv );

/*	gchar *text = NULL; */

	status_icon = dropline_status_icon_new();
	gtk_status_icon_set_visible(status_icon, TRUE);

/*	gtk_status_icon_set_tooltip(status_icon, "dropline GNOME applet is running."); */

	g_signal_connect(G_OBJECT(status_icon), "popup-menu",
			G_CALLBACK(dlg_popup), NULL); 

	g_signal_connect(G_OBJECT(status_icon), "activate",
			G_CALLBACK(dlg_check_for_updates), NULL); 

	g_signal_connect(G_OBJECT(status_icon), "activate",
			G_CALLBACK(dlg_preferences_window), NULL); 

	g_signal_connect(G_OBJECT(window), "button-press-event",
			G_CALLBACK(dlg_about_window), (gpointer) window); 

	/* Here we create a notification signal */
	g_signal_connect(G_OBJECT(dlg_menu), "activate",
			G_CALLBACK(dlg_notify_cb), NULL);

	gtk_main();

	return 0;
}
